@@ -20,6 +20,7 @@ from platform import platform
 from dateutil.relativedelta import relativedelta
 
 """ custom """
+from .tcex_job import TcExJob
 
 
 class TcEx(object):
@@ -35,7 +36,7 @@ class TcEx(object):
 
         self._exit_code = 0
         # TODO: replace group_type with dynamic values from API (bcs)
-        self.group_types = ['Adversary', 'Document', 'Email', 'Incident', 'Signature', 'Threat']
+        self.group_types = ['Adversary', 'Campaign', 'Document', 'Email', 'Incident', 'Signature', 'Threat']
         self.indicator_types = []
         self._max_message_length = 255
         # NOTE: odd issue where args is not updating properly
@@ -110,7 +111,6 @@ class TcEx(object):
 
     def _jobs(self):
         """Include jobs Module"""
-        from .tcex_job import TcExJob
         self.jobs = TcExJob(self)
 
     def _log_app_version(self):
@@ -208,7 +208,7 @@ class TcEx(object):
 
         if self._args.tc_token is not None and self._args.tc_log_to_api:
             # api & file logger
-            from api_logging_handler import ApiLoggingHandler
+            from .api_logging_handler import ApiLoggingHandler
             api = ApiLoggingHandler(logfile, self)
             api.set_name('api')  # not supported in python 2.6
             api.setLevel(level)
@@ -231,7 +231,7 @@ class TcEx(object):
         .. Note:: Playbook methods can be accessed using ``tcex.playbook.<method>``.
         """
         try:
-            from tcex_playbook import TcExPlaybook
+            from .tcex_playbook import TcExPlaybook
             self.playbook = TcExPlaybook(self)
         except ImportError as e:
             warn = 'Required playbook python dependency is not installed ({}).'.format(e)
@@ -460,7 +460,7 @@ class TcEx(object):
             (instance): An instance of DataFilter Class
         """
         try:
-            from tcex_data_filter import DataFilter
+            from .tcex_data_filter import DataFilter
             return DataFilter(self, data)
         except ImportError as e:
             err = 'Required Module is not installed ({}).'.format(e)
@@ -510,6 +510,10 @@ class TcEx(object):
         else:
             self.log.error('Invalid exit code')
 
+    def job(self):
+        """Return instance of Job module"""
+        return TcExJob(self)
+
     def message_tc(self, message):
         """Write data to message_tc file in TcEX specified directory.
 
@@ -565,13 +569,15 @@ class TcEx(object):
 
             if (self._args.tc_proxy_username is not None and
                     self._args.tc_proxy_password is not None):
+                tc_proxy_username = urllib.quote(self._args.tc_proxy_username, safe='~')
+                tc_proxy_password = urllib.quote(self._args.tc_proxy_password, safe='~')
 
                 proxies = {
                     'http': 'http://{0!s}:{1!s}@{2!s}:{3!s}'.format(
-                        self._args.tc_proxy_username, self._args.tc_proxy_password,
+                        tc_proxy_username, tc_proxy_password,
                         self._args.tc_proxy_host, self._args.tc_proxy_port),
                     'https': 'https://{0!s}:{1!s}@{2!s}:{3!s}'.format(
-                        self._args.tc_proxy_username, self._args.tc_proxy_password,
+                        tc_proxy_username, tc_proxy_password,
                         self._args.tc_proxy_host, self._args.tc_proxy_port)
                 }
             else:
@@ -596,7 +602,7 @@ class TcEx(object):
             (instance): An instance of Request Class
         """
         try:
-            from tcex_request import TcExRequest
+            from .tcex_request import TcExRequest
             return TcExRequest(self)
         except ImportError as e:
             err = 'Required Module is not installed ({}).'.format(e)
@@ -612,10 +618,12 @@ class TcEx(object):
         Returns:
             (instance): An instance of Request Class
         """
+        r = self.request
         if self._args.tc_proxy_external:
             self.log.info(u'Using proxy server for external request {}:{}.'.format(
                 self._args.tc_proxy_host, self._args.tc_proxy_port))
-        return self.request
+            r.proxy = self.proxy
+        return r
 
     def request_tc(self):
         """Return an instance of the Request Class with Proxy Set
@@ -625,10 +633,12 @@ class TcEx(object):
         Returns:
             (instance): An instance of Request Class
         """
+        r = self.request
         if self._args.tc_proxy_tc:
             self.log.info(u'Using proxy server for TC request {}:{}.'.format(
                 self._args.tc_proxy_host, self._args.tc_proxy_port))
-        return self.request
+            r.proxy = self.proxy
+        return r
 
     def resource(self, resource_type):
         """Get instance of Resource Class with dynamic group.
@@ -698,20 +708,37 @@ class TcEx(object):
             (string): Return decoded data
 
         """
-        if data is None or isinstance(data, (int, list, dict)):
-            pass  # do nothing with these types
-        elif isinstance(data, unicode):
-            try:
-                data.decode('utf-8')
-            except UnicodeEncodeError as e:  # 2to3 converts unicode to str
-                data = unicode(data.encode('utf-8').strip(), errors=errors)  # 2to3 converts unicode to str
-                self.log.warning('Encoding poorly encoded string ({})'.format(data))
-            except AttributeError:
-                pass  # Python 3 can't decode a str
-        else:
-            data = unicode(data, 'utf-8', errors=errors)
+        try:
+            if data is None or isinstance(data, (int, list, dict)):
+                pass # Do nothing with these types
+            elif isinstance(data, unicode):
+                try:
+                    data.decode('utf-8')
+                except UnicodeEncodeError as e:  # 2to3 converts unicode to str
+                    data = unicode(data.encode('utf-8').strip(), errors=errors)  # 2to3 converts unicode to str
+                    self.log.warning('Encoding poorly encoded string ({})'.format(data))
+                except AttributeError:
+                    pass  # Python 3 can't decode a str
+            else:
+                data = unicode(data, 'utf-8', errors=errors)  # 2to3 converts unicode to str
+        except NameError:
+            pass # Can't decode str in Python 3
         return data
 
+        ## if data is None or isinstance(data, (int, list, dict)):
+        ##     pass  # do nothing with these types
+        ## elif isinstance(data, unicode):
+        ##     try:
+        ##         data.decode('utf-8')
+        ##     except UnicodeEncodeError as e:  # 2to3 converts unicode to str
+        ##         data = unicode(data.encode('utf-8').strip(), errors=errors)  # 2to3 converts unicode to str
+        ##         self.log.warning('Encoding poorly encoded string ({})'.format(data))
+        ##     except AttributeError:
+        ##         pass  # Python 3 can't decode a str
+        ## else:
+        ##     data = unicode(data, 'utf-8', errors=errors)
+        ## return data
+
     def safe_indicator(self, indicator, errors='strict'):
         """Indicator encode value for safe HTTP request
 
