@@ -61,7 +61,7 @@ class Resource(object):
         if len(filters) > 0:
             self._r.add_payload('filters', ','.join(filters))
 
-    def _copy(self):
+    def copy(self):
         """Return a "clean" copy of this instance.
 
         Return:
@@ -136,6 +136,8 @@ class Resource(object):
                 data, status = self._request_process_json(response)
             elif response.headers['content-type'] == 'application/octet-stream':
                 data, status = self._request_process_octet(response)
+            elif response.headers['content-type'] == 'text/plain':
+                data, status = self._request_process_text(response)
             else:
                 err = 'Failed Request: {}'.format(response.text)
                 self._tcex.log.error(err)
@@ -250,6 +252,21 @@ class Resource(object):
 
         return data, status
 
+    def _request_process_text(self, response):
+        """Handle Signature download.
+
+        Return:
+            (string): The data from the download
+            (string): The status of the download
+        """
+        status = 'Failure'
+        # Handle document download
+        data = response.content
+        if len(data) != 0:
+            status = 'Success'
+
+        return data, status
+
     def add_filter(self, name, operator, value):
         """Add ThreatConnect API Filter for this resource request.
 
@@ -270,6 +287,17 @@ class Resource(object):
             'value': value
         })
 
+    def add_payload(self, key, val):
+        """Add a key value pair to payload for this request.
+
+        .. Note:: For ``_search`` you can pass a search argument. (e.g. _search?summary=1.1.1.1).
+
+        Args:
+            key (string): The payload key
+            val (string): The payload value
+        """
+        self._r.add_payload(key, val)
+
     @property
     def api_branch(self):
         """The ThreatConnect API branch for this resource.
@@ -325,7 +353,7 @@ class Resource(object):
             resource_api_branch (string): The resource pivot api branch.
             association_name (string): The name of the custom association as defined in the UI.
         """
-        resource = self._copy()
+        resource = self.copy()
         resource._request_uri = '{}/associations/{}/{}'.format(
             resource_api_branch, association_name, resource._request_uri)
         return resource
@@ -357,7 +385,7 @@ class Resource(object):
         Args:
             resource_api_branch (string): The resource pivot api branch including resource id.
         """
-        resource = self._copy()
+        resource = self.copy()
         resource._request_uri = '{}/{}'.format(
             association_resource.request_uri, resource._request_uri)
         return resource
@@ -389,7 +417,7 @@ class Resource(object):
         Return:
             (instance): A copy of this resource instance cleaned and updated for associations.
         """
-        resource = self._copy()
+        resource = self.copy()
         resource._request_entity = association_resource.api_entity
         resource._request_uri = '{}/{}'.format(
             resource._request_uri, association_resource.request_uri)
@@ -427,7 +455,7 @@ class Resource(object):
         Args:
             resource_id (Optional [string]): The resource id (attribute id).
         """
-        resource = self._copy()
+        resource = self.copy()
         resource._request_entity = 'attribute'
         resource._request_uri = '{}/attributes'.format(resource._request_uri)
         if resource_id is not None:
@@ -505,7 +533,7 @@ class Resource(object):
             resource_id (string): The resource pivot id (file hash).
             action_name (string): The name of the action as defined by ThreatConnect.
         """
-        resource = self._copy()
+        resource = self.copy()
         resource._request_uri = '{}/{}/actions/{}/{}'.format(
             resource._request_api_branch, resource_id, action_name, resource._request_uri)
         return resource
@@ -540,7 +568,7 @@ class Resource(object):
             (instance): A copy of this resource instance cleaned and updated for group associations.
 
         """
-        resource = self._copy()
+        resource = self.copy()
         resource._request_uri = '{}/{}'.format(group_resource.request_uri, resource._request_uri)
         return resource
 
@@ -590,7 +618,7 @@ class Resource(object):
             resource_type (string): The resource pivot resource type (indicator type).
             resource_id (integer): The resource pivot id (indicator value).
         """
-        resource = self._copy()
+        resource = self.copy()
         resource._request_uri = '{}/{}'.format(
             indicator_resource.request_uri, resource._request_uri)
         return resource
@@ -791,7 +819,7 @@ class Resource(object):
         Args:
             resource_id (string): The resource pivot id (security label name).
         """
-        resource = self._copy()
+        resource = self.copy()
         resource._request_uri = '{}/{}'.format(
             security_label_resource.request_uri, resource._request_uri)
         return resource
@@ -825,7 +853,7 @@ class Resource(object):
         Args:
             resource_id (Optional [string]): The resource id (security label name).
         """
-        resource = self._copy()
+        resource = self.copy()
         resource._request_entity = 'securityLabel'
         resource._request_uri = '{}/securityLabels'.format(resource._request_uri)
         if resource_id is not None:
@@ -859,7 +887,7 @@ class Resource(object):
         Args:
             resource_id (string): The resource pivot id (tag name).
         """
-        resource = self._copy()
+        resource = self.copy()
         resource._request_uri = '{}/{}'.format(
             tag_resource.request_uri, resource._request_uri)
         return resource
@@ -900,7 +928,7 @@ class Resource(object):
         Args:
             resource_id (Optional [string]): The resource id (tag name).
         """
-        resource = self._copy()
+        resource = self.copy()
         resource._request_entity = 'tag'
         resource._request_uri = '{}/tags'.format(resource._request_uri)
         if resource_id is not None:
@@ -931,7 +959,7 @@ class Resource(object):
         Args:
             resource_id (integer): The resource pivot id (task id).
         """
-        resource = self._copy()
+        resource = self.copy()
         resource._request_uri = '{}/{}'.format(
             task_resource.request_uri, resource._request_uri)
         return resource
@@ -945,6 +973,39 @@ class Resource(object):
         """
         return self._value_fields
 
+    def victims(self, victim_resource):
+        """Pivot point on Victims for this resource.
+
+        This method will return all *resources* (group, indicators, task,
+        etc) for this resource that are associated with the provided victim id.
+
+        **Example Endpoints URI's**
+
+        +--------------+-----------------------------------------------------------------------------------------------------+
+        | HTTP Method  | API Endpoint URI's                                                                                  |
+        +==============+=====================================================================================================+
+        | GET          | /v2/{resourceId}/groups/{resourceType}/{uniqueId}/victims                                           |
+        +--------------+-----------------------------------------------------------------------------------------------------+
+        | GET          | /v2/{resourceId}/groups/{resourceType}/{uniqueId}/victims/{victimId}                                |
+        +--------------+-----------------------------------------------------------------------------------------------------+
+        | GET          | /v2/{resourceId}/indicators/{resourceType}/{uniqueId}/victims                                       |
+        +--------------+-----------------------------------------------------------------------------------------------------+
+        | GET          | /v2/{resourceId}/indicators/{resourceType}/{uniqueId}/victims/{victimId}                            |
+        +--------------+-----------------------------------------------------------------------------------------------------+
+        | DELETE       | /v2/{resourceId}/groups/{resourceType}/{uniqueId}/victims/{victimId}                                |
+        +--------------+-----------------------------------------------------------------------------------------------------+
+        | POST         | /v2/{resourceId}/groups/{resourceType}/{uniqueId}/victims/{victimId}                                |
+        +--------------+-----------------------------------------------------------------------------------------------------+
+
+        Args:
+            resource_id (integer): The resource pivot id (victim id).
+        """
+        resource = self.copy()
+        resource._request_entity = 'victim'
+        resource._request_uri = '{}/{}'.format(
+            resource._request_uri, victim_resource.request_uri)
+        return resource
+
     def victim_pivot(self, victim_resource):
         """Pivot point on Victims for this resource.
 
@@ -968,11 +1029,70 @@ class Resource(object):
         Args:
             resource_id (integer): The resource pivot id (victim id).
         """
-        resource = self._copy()
+        resource = self.copy()
         resource._request_uri = '{}/{}'.format(
             victim_resource.request_uri, resource._request_uri)
         return resource
 
+    def victim_assets(self, asset_type=None, asset_id=None):
+        """Victim Asset endpoint for this resource with optional asset type.
+
+        This method will set the resource endpoint for working with Victim Assets.
+        The HTTP GET method will return all Victim Assets associated with this
+        resource or if a asset type is provided it will return the provided asset
+        type if it has been associated. The provided asset type can be associated
+        to this resource using the HTTP POST method.  The HTTP DELETE method will
+        remove the provided tag from this resource.
+
+        **Example Endpoints URI's**
+
+        +--------------+-----------------------------------------------------------------------------------------------------+
+        | HTTP Method  | API Endpoint URI's                                                                                  |
+        +==============+=====================================================================================================+
+        | GET          | /v2/groups/{resourceType}/{uniqueId}/victimAssets                                                   |
+        +--------------+-----------------------------------------------------------------------------------------------------+
+        | GET          | /v2/groups/{resourceType}/{uniqueId}/victimAssets/{assetType}                                       |
+        +--------------+-----------------------------------------------------------------------------------------------------+
+        | GET          | /v2/groups/{resourceType}/{uniqueId}/victimAssets/{assetType}/{resourceId}                          |
+        +--------------+-----------------------------------------------------------------------------------------------------+
+        | GET          | /v2/indicators/{resourceType}/{uniqueId}/victimAssets                                               |
+        +--------------+-----------------------------------------------------------------------------------------------------+
+        | GET          | /v2/indicators/{resourceType}/{uniqueId}/victimAssets/{assetType}                                   |
+        +--------------+-----------------------------------------------------------------------------------------------------+
+        | GET          | /v2/indicators/{resourceType}/{uniqueId}/victimAssets/{assetType}/{resourceId}                      |
+        +--------------+-----------------------------------------------------------------------------------------------------+
+        | GET          | /v2/victim/{uniqueId}/victimAssets/{assetType}                                                      |
+        +--------------+-----------------------------------------------------------------------------------------------------+
+        | GET          | /v2/victim/{uniqueId}/victimAssets/{assetType}/{resourceId}                                         |
+        +--------------+-----------------------------------------------------------------------------------------------------+
+        | DELETE       | /v2/groups/{resourceType}/{uniqueId}/victimAssets/{assetType}/{resourceId}                          |
+        +--------------+-----------------------------------------------------------------------------------------------------+
+        | POST         | /v2/groups/{resourceType}/{uniqueId}/victimAssets/{assetType}/{resourceId}                          |
+        +--------------+-----------------------------------------------------------------------------------------------------+
+
+        Args:
+            asset_type (Optional [string]): The asset type.
+            asset_id (Optional [string]): The asset id.
+        """
+        type_entity_map = {
+            'emailAddresses': 'victimEmailAddress',
+            'networkAccounts': 'victimNetworkAccount',
+            'phoneNumbers': 'victimPhone',
+            'socialNetworks': 'victimSocialNetwork',
+            'webSites': 'victimWebSite'
+        }
+        resource = self.copy()
+        resource._request_entity = 'victimAsset'
+        resource._request_uri = '{}/victimAssets'.format(resource._request_uri)
+        if asset_type is not None:
+            resource._request_entity = type_entity_map.get(asset_type, 'victimAsset')
+            resource._request_uri = '{}/{}'.format(
+                resource._request_uri, asset_type)
+            if asset_id is not None:
+                resource._request_uri = '{}/{}'.format(
+                    resource._request_uri, asset_id)
+        return resource
+
     @property
     def url(self):
         """The ThreatConnect URL for this resource request.
@@ -1138,6 +1258,10 @@ class Indicator(Resource):
         """
         return self.indicator_body(data)
 
+    def false_positive(self):
+        """Report indicator False Positive"""
+        self._request_uri = '{}/falsePositive'.format(self._request_uri)
+
     def indicator(self, data):
         """Update the request URI to include the Indicator for specific indicator retrieval.
 
@@ -1268,6 +1392,25 @@ class Indicator(Resource):
                         'value': indicator_data.get(indicator_field)
                     }
 
+    def observations(self):
+        """Report indicator observations"""
+        self._request_entity = 'observation'
+        self._request_uri = '{}/observations'.format(self._request_uri)
+
+    def observation_count(self):
+        """Retrieve indicator observation count"""
+        self._request_entity = 'observationCount'
+        self._request_uri = '{}/observationCount'.format(self._request_uri)
+
+    def observed(self, date_observed=None):
+        """Retrieve indicator observations count for top 10"""
+        if self.name != 'Indicator':
+            self._tcex.log.warning('Observed endpoint only available for "indicator" endpoint.')
+        else:
+            self._request_uri = '{}/observed'.format(self._request_uri)
+            if date_observed is not None:
+                self.r.add_payload('dateObserved', date_observed)
+
     def resource_id(self, data):
         """Alias for indicator method.
 
@@ -1329,7 +1472,7 @@ class Bulk(Indicator):
         """Initialize default class values."""
         super(Bulk, self).__init__(tcex)
         self._api_branch = 'bulk'
-        self._api_entity = 'indicator'
+        self._api_entity = 'bulkStatus'
         self._api_uri = '{}/{}'.format(self._api_branch_base, self._api_branch)
         self._name = 'Indicator'  # bcs - should this be bulk?
         self._request_entity = self._api_entity
@@ -1359,6 +1502,7 @@ class Bulk(Indicator):
         Args:
             ondemand (boolean): Enable on demand bulk generation.
         """
+        self._api_entity = 'indicator'
         self._request_uri = '{}/{}'.format(self._api_uri, 'json')
         self._stream = True
         if ondemand:
@@ -1418,6 +1562,9 @@ class File(Indicator):
         }
         body = {}
         for indicator in indicators:
+            if indicator is None:
+                continue
+
             if hash_patterns['md5'].match(indicator):
                 body['md5'] = indicator
             elif hash_patterns['sha1'].match(indicator):
@@ -1427,14 +1574,16 @@ class File(Indicator):
 
         return body
 
-    def occurrence(self, indicator):
+    def occurrence(self, indicator=None):
         """Update the URI to retrieve file occurrences for the provided indicator.
 
         Args:
             indicator (string): The indicator to retrieve file occurrences.
         """
         self._request_entity = 'fileOccurrence'
-        self._request_uri = '{}/{}/fileOccurrences'.format(self._api_uri, indicator)
+        self._request_uri = '{}/fileOccurrences'.format(self._request_uri)
+        if indicator is not None:
+            self._request_uri = '{}/{}/fileOccurrences'.format(self._api_uri, indicator)
 
 
 class Host(Indicator):
@@ -1456,6 +1605,17 @@ class Host(Indicator):
         self._request_uri = self._api_uri
         self._value_fields = ['hostName']
 
+    def resolution(self, indicator=None):
+        """Update the URI to retrieve host resolutions for the provided indicator.
+
+        Args:
+            indicator (string): The indicator to retrieve resolutions.
+        """
+        self._request_entity = 'dnsResolution'
+        self._request_uri = '{}/dnsResolutions'.format(self._request_uri)
+        if indicator is not None:
+            self._request_uri = '{}/{}/dnsResolutions'.format(self._api_uri, indicator)
+
 
 class URL(Indicator):
     """URL Resource Class
@@ -1516,7 +1676,7 @@ class Group(Resource):
             resource_id (string): The group id.
         """
         if self._name != 'group':
-            self._request_uri = '{}/{}'.format(self._request_uri, resource_id)
+            self._request_uri = '{}/{}'.format(self._api_uri, resource_id)
 
     def resource_id(self, resource_id):
         """Alias for group_id method
@@ -1675,6 +1835,14 @@ class Signature(Group):
         self._request_entity = self._api_entity
         self._request_uri = self._api_uri
 
+    def download(self, resource_id):
+        """Update the request URI to download the document for this resource.
+
+        Args:
+            resource_id (integer): The group id.
+        """
+        self.resource_id(str(resource_id))
+        self._request_uri = '{}/download'.format(self._request_uri)
 
 class Threat(Group):
     """Threat Resource Class
@@ -1906,7 +2074,7 @@ class Victim(Resource):
         self._status_codes = {
             'DELETE': [200],
             'GET': [200],
-            'POST': [200],
+            'POST': [200, 201],
             'PUT': [200]
         }
         self._value_fields = ['name']
